/*
 * mbintsvc.h
 *
 * This include file contains the interrupt service routines
 * 
 * Created 07/25/94 from the mboard.h file.
 * Updated 07/26/94 to use the interrupt_handler pragma
 * 	   08/04/94 Added ic1_svc, ic2_svc, do_userint().
 *		ic1/ic2 are used for encoders.
 *		do_userint() is used for pid drivers.
 */

/*
 * This is a kind of really gross hack, but it has tremendous value on
 * the limited memory of the miniboard. In this file I've put the 
 * interrupt service routines for the motors, beeper, and servos. I've
 * also put in "stub" routines. If you define variables WANT_xxx where
 * xxx is SERVOS, MOTORS, or BEEPER then the "real" interrupt service
 * routine gets compiled in, otherwise a stub routine gets compiled in
 * that saves space. This allows you to write a program that doesn't
 * need the servos for example and not have to carry around the code
 * for controlling them.
 */


#ifdef WANT_MOTORS
/* Motor variables */
unsigned char	motor_ctl;	/* Motor control high nybble on/off, low nybble dir */
unsigned int	speed[4];	/* Motor speed masks */
#endif

#ifdef WANT_BEEPER
/* beeper variable */
unsigned int	beeptone;	/* Tone sounding on the beeper */
#endif

#ifdef WANT_SERVOS
/* servo variables */
unsigned int	servo_position[3];	/* MAX of 3 servos */
#endif

#ifdef WANT_BUTTON
char	irqval;		/* IRQ button value */
int	buttime;	/* milleseconds */
#endif

/*
 * initialize requested options. Note this is called from mb_init() before
 * you're in "main()" so don't try to be clever and put your own stuff here.
 * Note we only declare variables for the options that have been selected.
 */
void
opt_init()
{
#ifdef WANT_MOTORS
	motor_ctl = 0;
	speed[0] = speed[1] = speed[2] = speed[3] = -1;
#endif

#ifdef WANT_BEEPER
	beeptone = 0;
#endif

#ifdef WANT_SERVOS
	servo_position[0] = servo_position[1] = servo_position[2] = 0;
#endif
#ifdef WANT_BUTTON
	irqval = buttime = 0;
#endif

#ifdef USER_INIT
	user_init();
#endif
	return;
}

#ifdef WANT_SERVOS
struct servo_param {
	unsigned int	neutral_value;
	unsigned char	step_value;
};

#ifndef SERVO1_NEUTRAL
#define SERVO1_NEUTRAL 	3050	/* Defines 1525 uS neutral position */
#endif
#ifndef SERVO1_STEP
#define SERVO1_STEP	17	/* Defines step size */
#endif
#ifndef SERVO2_NEUTRAL
#define SERVO2_NEUTRAL 	3050	/* Defines 1525 uS neutral position */
#endif
#ifndef SERVO2_STEP
#define SERVO2_STEP	17	/* Defines step size */
#endif
#ifndef SERVO3_NEUTRAL
#define SERVO3_NEUTRAL 	3050	/* Defines 1525 uS neutral position */
#endif
#ifndef SERVO3_STEP
#define SERVO3_STEP	17	/* Defines step size */
#endif

const struct servo_param parms[3] = {
	{ SERVO1_NEUTRAL, SERVO1_STEP },
	{ SERVO2_NEUTRAL, SERVO2_STEP },
	{ SERVO3_NEUTRAL, SERVO3_STEP }
};

/*
 * If requested, compile in the servo interrupt service handler routine.
 *
 * Position up to three servos on OC1, OC2, and OC3 (OC4 is used as the
 * system tick interrupt). I've written this code a couple of times and
 * this seems to be the most space efficient, in part because accessing
 * OC1 stuff versus OC2/OC3 stuff is asymmetric. The code trys to not
 * disturb any other bits in TCTL1 if a particular servo isn't being
 * used so that the user can do whatever they want with the pin if it
 * isn't driving a servo.
 */ 
void
do_servos()
{

	if (servo_position[0]) {
		asm("	ldx	#0x1000");	/* Point at I/O */
		asm("	ldaa	#0x80");	/* Forcing stuff */
		asm("	staa	0x0c,x");	/* OC1M = 0x80 */
		asm("	staa	0x0d,x");	/* OC1D = 0x80 */
		asm("	staa	0x0b,x");	/* Force it */
		asm("	clra	");		/* Clear A */
		asm("	staa	0x0d,x");	/* Clear the bit */
		asm("	ldd	0x0e,x");	/* Setup to switch it */
		asm("	addd	_servo_position");
		asm("	std	0x16,x");	/* At T + n mSec */
		asm("	tsx");			/* Restore X */
	}
	if (servo_position[1]) {
		/* 
		 * Check this out, we set both bits in TCTL1 which
		 * tell the system that effect of a compare is to
		 * set the pin to '1', then we reset just one bit
		 * changing TCTL1 so that the operation on compare is
		 * to clear the pin.
		 */
		asm("	ldx	#0x1000");	/* Point at I/O */
		asm("	bset	0x20,x,#0xc0");	/* TCTL1 = 0xc0 */
		asm("	ldaa	#0x40");	/* Force only OC2 */
		asm("	staa	0x0b,x");	/* Force it */
		asm("	bclr	0x20,x,#0x40");	/* Clear bit */
		asm("	ldd	0x0e,x");	
		asm("	addd	_servo_position+2");
		asm("	std	0x18,x");
		asm("	tsx");			/* Restore X */
	}
	if (servo_position[2]) {
		asm("	ldx	#0x1000");	/* Point at I/O */
		asm("	bset	0x20,x,#0x30");	/* TCTL1 = 0x30 */
		asm("	ldaa	#0x20");	/* Force only OC3 */
		asm("	staa	0x0b,x");	/* Force it */
		asm("	bclr	0x20,x,#0x10");	/* Clear bit */
		asm("	ldd	0x0e,x");	
		asm("	addd	_servo_position+4");
		asm("	std	0x1a,x");
		asm("	tsx");			/* Restore X */
	}
	return;
}
#else
/* if the interrupt handler isn't selected, compile in this stub routine */
void do_servos() { return; }
#endif

#ifdef WANT_MOTORS
/*
 * If motors are selected, then compile in the PWM generator function.
 *
 * This is the PWM function for the motor. It uses PORT B but could be
 * easily modified to use a different port. Since it clocks out 1 bit
 * every system tick (1Khz system tick) the frequency at 50% is 500 hz.
 */
void
do_motors()
{
	unsigned char 	motor_mask, 
			motors;
	int		i,j;

	if ((motor_ctl & 0xf0) == 0) {
		PORTB = 0; /* eliminates race condition by forcing off */
		return;	/* nothing to do */
	}
	motor_mask = 0x10;	/* Mask information */
	motors = motor_ctl;

	/*
	 * For each motor we check to see if it is enabled in
 	 * the motor_ctl variable, and if it is we then take
	 * its "speed" bitmask and effectively do a rotate right
	 * on it. We preserve the state of the low order bit so
	 * that we can add it back in after the shift (thus making
	 * it a rotation).
	 */
#ifdef BEEP_ON_MOTOR4
	motors |= PORTB & 0x80; /* get current beeper state */
	for (i = 0; i < 3; i++, motor_mask = motor_mask << 1) {
#else
	for (i = 0; i < 4; i++, motor_mask = motor_mask << 1) {
#endif
		if ((motor_ctl & motor_mask) == 0)
			continue;
		j = speed[i] & 0x1; /* check the bit */
		speed[i] = speed[i] >> 1; /* shift speed */
		if (j)
			speed[i] |= 0x8000; /* complete the rotate */
		else
			motors &= ~motor_mask; /* turn off the motor */
	}
	PORTB = motors;	/* actually do it */
	return;
}
#else
/* else if motors aren't selected then compile in this stub routine */
void do_motors() { return; }
#endif

#pragma interrupt_handler beep_svc()

#ifdef WANT_BEEPER
/*
 * The beeper's interrupt service routine. Simply reload the OC register
 * with the half/cycle time constant to flip the output bit.
 */

void
beep_svc()
{
	asm("	ldx	#0x1000");
	asm("	ldd	0x0e,x");	/* get TCNT */
	asm("	addd	_beeptone");	/* Add value of beeptone */
	asm("	std	0x1e,x");	/* Store it back */
	asm("	bclr	0x23,x,#0xf7");
#ifdef BEEP_ON_MOTOR4
	asm("	ldaa	0x04,x");	/* get current value */
	asm("	eora	#8");		/* change the direction */
	asm("	staa	0x04,x");	/* store it back */
#endif
}
#else
/* if the beeper wasn't selected then compile in this stub function */
void beep_svc() { }
#endif

#pragma interrupt_handler irqi_svc()

#ifdef WANT_BUTTON
/* The interrupt service routine that deals with the IRQ button */
void
irqi_svc()
{
	/* Debounce the switch */
	if ((LO_WORD(time) - buttime) < 60)
		return;
	irqval = 1;
	buttime = LO_WORD(time);
}
#else
/* stub routine for the button */
void irqi_svc() { }
#endif

#pragma interrupt_handler ic1_svc()
#ifndef IC1_SVC
/* stub routine for ic1_svc() */
void ic1_svc() { }
#endif

#pragma interrupt_handler ic2_svc()
#ifndef IC2_SVC
/* stub routine for ic2_svc() */
void ic2_svc() { }
#endif

#ifndef DOUSERINT
void do_userint() { }
#endif
