/*
 * miniboard.h
 *
 * Created 7/20/94 by C. McManis
 *
 * Version history:
 *	07/20/94	created. -cmcmanis
 *	07/21/94	added "undocumented" functions and new ones
 *	07/22/94	put the interrupt service routines "in" here.
 *	07/23/94	added the IRQ service routine to the list.
 *	07/25/94	Created separate interrupt service file
 *			and added header guards to this file.
 *
 * This file is the "standard" include file for programs that use
 * the miniboard package. It defines the prototypes for the functions
 * available in the library and global variables your program may wish
 * to access. Note that if you don't use a function it won't be linked
 * into your program.
 */

#ifndef MBOARD_H
#define MBOARD_H

#include <int32.h>	/* Time is defined as a 32 bit quantity */

extern INT32 time;

#ifndef PORTB
#define PORTB	*(unsigned char *)(0x1004)
#endif

/*
 * Define offsets for the I/O Ports that we use, this should probably
 * be part of hc11.h but this is where it is for now.
 */
#define	TCTL2_OFF	0x21
#define TMSK1_OFF	0x22
#define	TFLG1_OFF	0x23
#define TMSK2_OFF	0x24
#define TFLG2_OFF	0x25
#define TIC1_OFF	0x10
#define TIC2_OFF	0x12
#define TIC3_OFF	0x14
#define	TIC4_OFF	0x1e

/*
 * analog() - read and return an analog value.
 */
extern int analog(int);

/*
 * digital() - read a bit from the digital I/O port, bit 0 is furthest
 * away from the PWR/RCV LEDs on the board. NB: The values are inverted
 * so reading 'digital(0)' will return FALSE if the signal at bit 0 is
 * "logic high". The reason for this is that it is more natural to read
 * this as "TRUE means the switch connected to bit 'n' is closed, FALSE
 * means that it is open."
 */
extern int digital(int);

/*
 * motor() - This function will turn on (or off) one of the motors.
 * The two parameters are motor number and speed. The motor number
 * should be between 1 and 4, the speed between -16 to 16. A speed of
 * 0 turns off the motor. Illegal values are ignored and the function
 * call becomes a nop.
 */
extern void motor(int, int);

/*
 * off() - Turn the named motor off. In some systems this is actually
 * a separate function, I found it however to simply take up space.
 * setting the speed to 0 is equivalent to older versions of off.
 */
#define off(m) motor(m, 0)

/*
 * button() - This function returns TRUE if the IRQ button has been pressed
 * since the last time it was called.
 */
extern int button(void);
 
/*
 * msleep() - This function provides a fairly accurate time delay of 'n'
 * milliseconds. It effectively goes into a spin loop however the tick
 * interrupt keeps the motor PWM stuff going so the motors still work.
 * Its accuracy can be disrupted by Tones that are playing.
 */
extern void msleep(int);

/*
 * tone(freq, dur) - Generate a tone on the Piezo beeper and optionally on
 * motor 4. 
 * NB: By default Motor 4 is _not_ used to mirror the beeper. If you want
 * the tone on motor for you must recompile tone.c with BEEP_ON_MOTOR4
 * defined. Also note this function calls msleep.
 */
extern void tone(int, int);

/*
 * printdec(num) - This function will print an unsigned representation of
 * the number 'num' with leading zeros and in 5 digits. It always prints
 * a <cr><lf> sequence after the number.
 */
extern void printdec(int);

/*
 * getchar() - This function will return a single character from the
 * serial port.
 */
extern unsigned char getchar(void);
#define getch	getchar	/* for compatibility with the documentation */

/* ===========================================================
 * Additional functions: These functions are not described in the Miniboard
 * 2.0 technical manual. They are documented here.
 * ===========================================================
 */

/*
 * servo(s, n) - This function will position one of 3 servos between
 * -100% and 100% of their travel. 0% is considered the neutral position.
 */
void servo(int, int);
/*
 * servo_off(s) - This function will "turn off" servo 1 through 3.
 */
void servo_off(int);

/*
 * putchar(c) - This function will print the character 'c' to the serial
 * port. By default it does this at 9600 baud, 8 bits, 1 stop bit, and
 * no parity.
 */
extern void putchar(unsigned char);

/* these typedef's save some typing and provide useful types */

typedef unsigned char u_char;
typedef unsigned int u_int;

/*
 * The next for #define's simulate the peek and poke functions from the
 * original library. They are much more space efficient since they convert
 * usually into a single instruction rather than a call to a function.
 */
#define peek(addr)	 	(*(u_char *)(addr))
#define poke(addr,value) 	(*(u_char *)(addr)) = (u_char)(value)
#define peekword(addr)	 	(*(u_int *)(addr))
#define pokeword(addr,value) 	(*(u_int *)(addr)) = (u_int)(value)

#endif /* ifndef __MBOARD_H__ */
