/*
 * tone.c
 *
 * Created 7/21/94 by Chuck McManis
 *
 * This module implements the tone function.
 */

#include <hc11.h>

extern unsigned int beeptone;

/* tctl1 = $1020 */
/* tmsk1 = $1022 */

/*
 * tone
 * 
 * The tone setup function. This function calculates the number of
 * "ticks" in a half cycle of the tone and stores that in beeptone.
 * It then enables interrupts for OC5 and when the timer "expires"
 * the OC5 pin's state is changed (ie a half cycle). But what about
 * that weird tick calculation? We want to calculate the number of
 * ticks but we've only got 16 bit integers so what to do? The
 * calculation is :
 *	2,000,000 ticks		1      second   1  cycles
 *	---------------	* ------------------- * -  -------
 *	    1     second      freq     cycles   2  half cycles
 * if you cancel all the units you'll see you end up with ticks/half cycles.
 * But event reducing the 2,000,000 to 1,000,000 (multiplied by .5 cyc/half)
 * is out of the unsigned 16 bit range of 65535. We use the trick that
 * you can always multiply by "1" and "1" can be represented by 16/16. That
 * means we divide 1,000,000 by 16 which gives us 62500 (representable in
 * 16 bits) and then divide by the freqency, then multiply the result by
 * 16 to get the delay constant. The "problem" is that this introduces some
 * error, but in this case its acceptable. 
 */
void
tone(t, d)
	int	t;	/* tone in Hz */
	int	d;	/* duration */
{
	unsigned int	x;

	x = (62500 / (unsigned int) t) << 4;  /* Calculate tone constant */
	beeptone = x;
	asm("	ldx	#0x1000");	/* Load X with IO_BASE */
	asm("	bset	0x20,x,#0x1");	/* TCTL1 -> beeper pin toggle */
	asm("	bset	0x22,x,#0x8");	/* enable beeper interrupt */
#ifdef BEEP_ON_MOTOR4
	asm("	bset	0x04,x,#0x80");	/* enable motor 4 */
#endif
	asm("	tsx");		/* get X back from stack */
	msleep(d);	/* idle for a while */
	asm("	ldx 	#0x1000");
	asm("	bclr	0x22,x,#0x8");	/* disable beeper interrupt */
	asm("	bclr	0x20,x,#0x1");	/* Turn of line toggling */
#ifdef BEEP_ON_MOTOR4
	asm("	bclr	0x04,x,#0x80");	/* disable motor 4 */
#endif
	asm("	tsx");			/* restore value of X */
	beeptone = 0;			/* clear beeptone */
	return;
}
